--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r184
Project Date		: 20090527181224

File				: SKG.lua
Commit Author		: zhinjio
Commit Revision		: 184
Commit Date			: 20090527181224
************************************************************************
Description	:
	Main addon primary methods (oninit, onenable, ondisable) and
	associated goo.

TODO		:
	Still need usage message

	Not sure why my timer frame code is in here. Is there maybe a better
	place for it?
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

local addon = SKG

-- Show Config options table
-- Needs to be defined here since Locales must be loaded, and the functions
-- referred to must pass dewdrop arg checks (as such, they must exist)
fubarOptionsTable = {
	name = "SuicideKingsGeo",
	desc = "SuicideKingsGeo",
	handler = SKG,
	type = 'group',
	args = {
		["GenOpts"] = {
			name = SKGL["SKG_LBL_OF_TITLE_TEXT"],
			desc = SKGL["SKG_LBL_OF_TITLE_TEXT"],
			type = 'group',
			args = {
				["raidonly"] = {
					name = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.raidonly
					end,
					order = 100,
					set = SKG_OF_RaidOnlyToggle,
				},
				["completelist"] = {
					name = SKGL["SKG_LBL_OF_SHOWALLCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_SHOWALLCB_TEXT"],
					type = 'toggle',
					get = function() 
						return not SKG.db.profile.Options.General.raidonly
					end,
					order = 110,
					set = SKG_OF_RaidOnlyToggle,
				},
				["displayclasscolors"] = {
					name = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.classColors
					end,
					order = 120,
					set = SKG_OF_ShowClassColorsClicked,
				},
				["spacer1"] = {
					type = "header",
					order = 130,
				},
				["autoopen"] = {
					name = SKGL["SKG_LBL_OF_AUTOOPENCB_TEXT"],
					desc = SKGL["SKG_OF_AUTOOPENCB_TT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.autoopen
					end,
					order = 140,
					set = SKG_OF_AutoOpenClicked,
				},
				["spacer2"] = {
					type = "header",
					order = 150,
				},
				["debug"] = {
					name = SKGL["SKG_LBL_OF_DEBUGCB_TEXT"],
					desc = SKGL["SKG_OF_DEBUGCB_TT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.debug
					end,
					order = 160,
					set = SKG_OF_DebugClicked,
				}
			}
		}
	}
}

-- Check to see if our SV database is up to date with current specs, etc.
function SKG:upgradeDB()
	local olddb = addon.db.profile
	if not olddb.Options.General.dbVersion then
		-- file existed before I added in db versioning. Do the right thing.
		olddb.Options.General.dbVersion = "1.0"
		SKG_Print( string.format( SKGL["Database upgraded to %s"], "1.0" ) )
	end
	if ( olddb.Options.General.dbVersion == "1.0" ) then
		-- get rid of now unused security policy blocks
		olddb.Options.Security.Policies = nil
		olddb.Options.Security = nil
		olddb.Options.General.dbVersion = "1.1"
		SKG_Print( string.format( SKGL["Database upgraded to %s"], "1.1" ) )
	end
	if ( olddb.Options.General.dbVersion == "1.1" ) then
		olddb.announce = 0
		olddb.Options.General.dbVersion = "1.2"
		SKG_Print( string.format( SKGL["Database upgraded to %s"], "1.2" ) )
	end
end

-- on init
function SKG:OnInitialize()
	-- Frame setup if needed
end

function SKG:OnEnable()
    -- Called when the addon is loaded
    self.db = LibStub( "AceDB-3.0" ):New( "SKG_DB", nil, "Default" )

	if not ( self.db.profile.firstLoad ) then
		sdbp = self.db.profile
		sdbp.firstLoad = true
		sdbp.announce = 0
		sdbp.Players = {}
		sdbp.Alts = {}
		sdbp.RollLists = {}
		sdbp.LastMod = {}
		sdbp.Log = {
			["SKG"] = {},
			["Loot"] = {},
		}
		sdbp.Snapshot = {}
		sdbp.SnapshotLM = {}
		sdbp.Options = {
			General = {
				debug = false,
				autoopen = false,
				buttonTexture = 1,
				classColors = true,
				raidonly = false,
				dbVersion = "1.2",
			},
			Bid = {
				ArmorFilter = 0,
			},
			List = {},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end

	-- Event listeners (handlers are in SKG_Events.lua)
	SKG:RegisterEvent( "ADDON_LOADED" )
	SKG:RegisterEvent( "LOOT_OPENED" )
	SKG:RegisterEvent( "RAID_ROSTER_UPDATE" )
	SKG:RegisterEvent( "CHAT_MSG_WHISPER" )
	SKG:RegisterEvent( "UNIT_SPELLCAST_SENT" )
	-- Other events to worry about
	-- PARTY_LOOT_METHOD_CHANGED
	SKG:RegisterEvent( "GUILD_ROSTER_UPDATE" )

	SKG_INIT_GUILD = 1

	-- Comm registration (handlers are in SKG_Comm.lua)
	SKG:RegisterComm( SKG_CHANNEL_PREFIX )

	-- Called when the addon is enabled
	local major, minor = SKG_getVersionComponents()
	SKG_Print( string.format( SKGL["SKG_MSG_ADDON_LOADED"], major, minor ) )
	
	-- Fire off a guild roster call. speed the progress along to getting out security
	-- info
	GuildRoster()
	
	-- Check to see if we're supposed to show an announcement
	if ( SKG_ANNOUNCE ~= 0 ) then
		-- We are, but have they seen it already?
		if ( self.db.profile.announce ~= SKG_ANNOUNCE ) then
			-- They haven't seen it. Spit it out
			local announceText = "SKG_ANNOUNCE_" .. SKG_ANNOUNCE
			StaticPopupDialogs["SKG_ANNOUNCE"].text = SKGL[ announceText ]
			StaticPopup_Show("SKG_ANNOUNCE")
		end
	end
end

function SKG:OnDisable()
    -- Called when the addon is disabled
	SKG_ENABLED = false
	-- do my best to make the addon disappear from view
	SKG_Frame:Hide()
	-- get rid of the chat command
	SKG:UnregisterChatCommand( "skg" )
end

function SKG:Disable()
    -- Called when the addon is disabled
	SKG_ENABLED = false
	-- do my best to make the addon disappear from view
	SKG_Frame:Hide()
	-- get rid of the chat command
	SKG:UnregisterChatCommand( "skg" )
end

-- Register our chat command, and set its function
SKG:RegisterChatCommand("skg", "OnSlashCommand")

-- And now do the dirty work
function SKG:OnSlashCommand( text )
	if ( SKG_ENABLED == true ) then
		local db = SKG.db.profile.Options.General
		if ( text == "" ) then
			if ( SKG_Frame:IsVisible() ) then
				PlaySound("igMainMenuClose");
				SKG_Frame:Hide()
			else
				PlaySound("igMainMenuOpen");
				SKG_Frame:Show()
			end
		elseif ( text == "reset" ) then
			db.xPos = 100
			db.yPos = -100
			SKG_Frame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 100, -100 )
			SKG_Frame:Show()
		elseif ( text == "checklists" ) then
			-- These commands should only work if you're a syncer (or above) 
			if ( SKG_SECURITY_LEVEL ~= "U" ) then
				SKG_Print( SKGL["SKG_MSG_DIAG"] )
				-- perform a diagnostic check on lists to see if they have corruption
				-- Check Players list
				local rc = SKG_checkList( "Players" )
				SKG_Print( SKG_ListError( "Players", rc ) )
				-- Check Alts list
				rc = SKG_checkList( "Alts" )
				SKG_Print( SKG_ListError( "Alts", rc ) )
				-- loop through the rolling lists
				local db = SKG.db.profile.RollLists
				for k, v in pairs( db ) do
					rc = SKG_checkList( k )
					SKG_Print( SKG_ListError( k, rc ) )
				end
				SKG_Print( SKGL["SKG_MSG_DIAGCOMPLETE"] )
			else
				SKG_Print( SKGL["SKG_ERROR_NOTLEET"] )
			end
		elseif ( text == "repairlists" ) then
			-- make an attempt to get rid of bad entries in lists
			if ( SKG_SECURITY_LEVEL ~= "U" ) then
				SKG_Print( SKGL["SKG_MSG_LISTREPAIR"] )
				SKG_RepairLists()
				SKG_Print( SKGL["SKG_MSG_REPAIRCOMPLETE"] )
			else
				SKG_Print( SKGL["SKG_ERROR_NOTLEET"] )
			end
		else
			SKG_Print( SKGL["SKG Usage:"] )
			SKG_Print( "  " .. SKGL["/skg - open SKG GUI"] )
			SKG_Print( "  " .. SKGL["/skg reset - reset SKG GUI position"] )
			SKG_Print( "  " .. SKGL["/skg checklists - check lists for corruption"] )
			SKG_Print( "  " .. SKGL["/skg repairlists - attempt to repair list corruption"] )
		end
	end
end

local LDB = LibStub( "LibDataBroker-1.1", true )
local launcher = LDB:NewDataObject( "SKG", {
	type = "launcher",
	icon = "Interface\\Icons\\INV_Elemental_Primal_Nether",
	OnClick = function(clickedframe, button)
		if ( button == "RightButton" ) then
			SKG_ShowConfig( clickedframe )
		else
			SKG_Toggle()
		end
	end,
	OnTooltipShow = function( tooltip )
		tooltip = tooltip or GameTooltip
		tooltip:AddLine( SKGL["SKG_LBL_OF_TITLE_TEXT"], 1, 1, 1 )
		tooltip:AddLine( " " )
		local db = SKG.db.profile.Options.General
		local text1 = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"]
		local text2 = ""
		if ( db.raidonly ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"]
		if ( db.classColors ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_AUTOOPENCB_TEXT"]
		if ( db.autoopen ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_DEBUGCB_TEXT"]
		if ( db.debug ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		tooltip:AddLine( " " )
		tooltip:AddLine( SKGL["SKG_LBL_LEFTCLICK"], 1, 1, 1 )
		tooltip:AddLine( SKGL["SKG_LBL_RIGHTCLICK"], 1, 1, 1 )
	end,		

} )

--[[
************************************************************************
CHANGELOG:

Date : 5/20/09
	Adding in "announce" code
Date : 4/23/09
	Modified the SV structure to remove the now unused security blocks, and changed
	upgradedb to remove the old blocks
Date : 10/26/08
	Moving RegisterEvent calls to OnInit
	Moving RegisterComm call to OnInit
	Defined fubarOptionsTable to be global
Date : 10/21/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer.
************************************************************************
]]--